/*
 * Rev. 0.2 EPSG/POSC Units Database.
 */
 
#ifdef INCLUDE_OLD_CODES
#include geo_units.inc
#endif /* OLD Codes */

ValuePair(Linear_Meter,	9001)
ValuePair(Linear_Foot,	9002)
ValuePair(Linear_Foot_US_Survey,	9003)
ValuePair(Linear_Foot_Modified_American,	9004)
ValuePair(Linear_Foot_Clarke,	9005)
ValuePair(Linear_Foot_Indian,	9006)
ValuePair(Linear_Link,	9007)
ValuePair(Linear_Link_Benoit,	9008)
ValuePair(Linear_Link_Sears,	9009)
ValuePair(Linear_Chain_Benoit,	9010)
ValuePair(Linear_Chain_Sears,	9011)
ValuePair(Linear_Yard_Sears,	9012)
ValuePair(Linear_Yard_Indian,	9013)
ValuePair(Linear_Fathom,	9014)
ValuePair(Linear_Mile_International_Nautical,	9015)
/*
 *  Angular Units
 */
ValuePair(Angular_Radian,	9101)
ValuePair(Angular_Degree,	9102)
ValuePair(Angular_Arc_Minute,	9103)
ValuePair(Angular_Arc_Second,	9104)
ValuePair(Angular_Grad,	9105)
ValuePair(Angular_Gon,	9106)
ValuePair(Angular_DMS,	9107)
ValuePair(Angular_DMS_Hemisphere,	9108)
/* end of list */
